@@ -34,6 +34,9 @@
 /* For >= 500 for pread/pwrite and readdir_r; >= 700 for utimensat */
 #define _XOPEN_SOURCE 700
 
+/* For flock() on FreeBSD. It otherwise gets hidden by _XOPEN_SOURCE  */
+#define __BSD_VISIBLE 1
+
 /* For stat() nanosecond precision and lutimes() */
 #define _BSD_SOURCE
 /* The new non-deprecated version of _BSD_SOURCE */
@@ -69,6 +72,10 @@
 #include <sys/xattr.h>
 #endif
 
+#ifdef __LINUX__
+#include <linux/fs.h>  // For BLKGETSIZE64
+#endif
+
 #include <fuse.h>
 #include <fuse_opt.h>
 
@@ -161,6 +168,8 @@ static struct Settings {
     int hide_hard_links;
     int resolve_symlinks;
 
+    int block_devices_as_files;
+
     enum ResolvedSymlinkDeletion {
         RESOLVED_SYMLINK_DELETION_DENY,
         RESOLVED_SYMLINK_DELETION_SYMLINK_ONLY,
@@ -362,8 +371,34 @@ static int getattr_common(const char *procpath, struct stat *stbuf)
     }
 
     /* Hide hard links */
-    if (settings.hide_hard_links)
+    if (settings.hide_hard_links) {
         stbuf->st_nlink = 1;
+    }
+
+    /* Block files as regular files. */
+    if (settings.block_devices_as_files && S_ISBLK(stbuf->st_mode)) {
+        stbuf->st_mode ^= S_IFBLK | S_IFREG;  // Flip both bits
+#ifdef __LINUX__
+        uint64_t size;
+        ioctl(file, BLKGETSIZE64, &size);
+        stbuf->st_size = (off_t)size;
+        if (stbuf->st_size < 0) {  // Underflow
+            return -EOVERFLOW;
+        }
+#else
+        int fd = open(procpath, O_RDONLY);
+        if (fd == -1) {
+            return -errno;
+        }
+        off_t size = lseek(fd, 0, SEEK_END);
+        if (size == (off_t)-1) {
+            close(fd);
+            return -errno;
+        }
+        stbuf->st_size = size;
+        close(fd);
+#endif
+    }
 
     /* Then permission bits. Symlink permissions don't matter, though. */
     if ((stbuf->st_mode & S_IFLNK) != S_IFLNK) {
@@ -1424,7 +1459,7 @@ static void print_usage(const char *progname)
            "  --write-rate=...          Limit to bytes/sec that can be written.\n"
            "\n"
            "Miscellaneous:\n"
-           "  -n      --no-allow-other  Do not add -o allow_other to fuse options.\n"
+           "  --no-allow-other          Do not add -o allow_other to fuse options.\n"
            "  --realistic-permissions   Hide permission bits for actions mounter can't do.\n"
            "  --ctime-from-mtime        Read file properties' change time\n"
            "                            from file content modification time.\n"
@@ -1433,6 +1468,7 @@ static void print_usage(const char *progname)
            "  --hide-hard-links         Always report a hard link count of 1.\n"
            "  --resolve-symlinks        Resolve symbolic links.\n"
            "  --resolved-symlink-deletion=...  Decide how to delete resolved symlinks.\n"
+           "  --block-devices-as-files  Show block devices as regular files.\n"
            "  --multithreaded           Enable multithreaded mode. See man page\n"
            "                            for security issue with current implementation.\n"
            "\n"
@@ -1475,7 +1511,7 @@ enum OptionKey {
     OPTKEY_ENABLE_IOCTL,
     OPTKEY_HIDE_HARD_LINKS,
     OPTKEY_RESOLVE_SYMLINKS,
-    OPTKEY_MULTITHREADED
+    OPTKEY_BLOCK_DEVICES_AS_FILES
 };
 
 static int process_option(void *data, const char *arg, int key,
@@ -1568,6 +1604,9 @@ static int process_option(void *data, const char *arg, int key,
     case OPTKEY_RESOLVE_SYMLINKS:
         settings.resolve_symlinks = 1;
         return 0;
+    case OPTKEY_BLOCK_DEVICES_AS_FILES:
+        settings.block_devices_as_files = 1;
+        return 0;
 
     case OPTKEY_NONOPTION:
         if (!settings.mntsrc) {
@@ -1669,12 +1708,12 @@ static int parse_user_map(UserMap *map, UserMap *reverse_map, char *spec)
             }
             *q = '\0';
             if (!group_gid(tmpstr + 1, &gid_from)) {
-                fprintf(stderr, "Invalid group: %s\n", tmpstr);
+                fprintf(stderr, "Invalid group: %s\n", tmpstr + 1);
                 goto fail;
             }
             q += strlen("/@");
             if (!group_gid(q, &gid_to)) {
-                fprintf(stderr, "Invalid group: %s\n", tmpstr);
+                fprintf(stderr, "Invalid group: %s\n", q);
                 goto fail;
             }
 
@@ -1703,7 +1742,7 @@ static int parse_user_map(UserMap *map, UserMap *reverse_map, char *spec)
             }
             q += strlen("/");
             if (!user_uid(q, &uid_to)) {
-                fprintf(stderr, "Invalid username: %s\n", tmpstr);
+                fprintf(stderr, "Invalid username: %s\n", q);
                 goto fail;
             }
 
@@ -1899,6 +1938,7 @@ int main(int argc, char *argv[])
         OPT2("--hide-hard-links", "hide-hard-links", OPTKEY_HIDE_HARD_LINKS),
         OPT2("--resolve-symlinks", "resolve-symlinks", OPTKEY_RESOLVE_SYMLINKS),
         OPT_OFFSET2("--resolved-symlink-deletion=%s", "resolved-symlink-deletion=%s", resolved_symlink_deletion, -1),
+        OPT2("--block-devices-as-files", "block-devices-as-files", OPTKEY_BLOCK_DEVICES_AS_FILES),
 
         OPT2("--realistic-permissions", "realistic-permissions", OPTKEY_REALISTIC_PERMISSIONS),
         OPT2("--ctime-from-mtime", "ctime-from-mtime", OPTKEY_CTIME_FROM_MTIME),
@@ -1947,6 +1987,7 @@ int main(int argc, char *argv[])
     settings.hide_hard_links = 0;
     settings.resolve_symlinks = 0;
     settings.resolved_symlink_deletion_policy = RESOLVED_SYMLINK_DELETION_SYMLINK_ONLY;
+    settings.block_devices_as_files = 0;
     settings.realistic_permissions = 0;
     settings.ctime_from_mtime = 0;
     settings.enable_lock_forwarding = 0;
@@ -2156,8 +2197,15 @@ int main(int argc, char *argv[])
     fuse_opt_add_arg(&args, "-ouse_ino");
     fuse_opt_add_arg(&args, "-oreaddir_ino");
 
-    /* Show the source dir in the first field on /etc/mtab. */
-    {
+    /* Show the source dir in the first field on /etc/mtab, to be consistent
+       with "real" filesystems.
+
+       We don't do this if the source dir contains some special characters.
+       Comma is on this list because it would mess up FUSE's option parsing
+       (issue #47). The character blacklist is likely not complete, which is
+       acceptable since this is not a security check. The aim is to avoid giving
+       the user a confusing error. */
+    if (strpbrk(settings.mntsrc, ", \t\n") == NULL) {
         char *tmp = sprintf_new("-ofsname=%s", settings.mntsrc);
         fuse_opt_add_arg(&args, tmp);
         free(tmp);
