@@ -18,6 +18,8 @@
 */
 
 #include "misc.h"
+#include <stdarg.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
@@ -68,6 +70,31 @@ char *strdup_until(const char *s, const char *endchars)
     }
 }
 
+char *sprintf_new(const char *format, ...)
+{
+    va_list ap;
+    size_t buffer_size = strlen(format) + 32;
+    char *buffer = malloc(buffer_size);
+    if (buffer == NULL) {
+        return NULL;
+    }
+
+    while (1) {
+        va_start(ap, format);
+        size_t len = (size_t)vsnprintf(buffer, buffer_size, format, ap);
+        va_end(ap);
+        if (len < buffer_size) {
+            return buffer;
+        }
+        free(buffer);
+        buffer_size *= 2;
+        buffer = malloc(buffer_size);
+        if (buffer == NULL) {
+            return NULL;
+        }
+    }
+}
+
 const char *my_basename(const char *path)
 {
     const char *p;
@@ -125,7 +152,7 @@ int parse_byte_count(const char *str, double *result)
 {
     char* end;
     double base = strtod(str, &end);
-    long mul = 1;
+    long long mul = 1;
     if (*end == '\0') {
         mul = 1L;
     } else if (strcmp(end, "k") == 0) {
@@ -135,7 +162,7 @@ int parse_byte_count(const char *str, double *result)
     } else if (strcmp(end, "G") == 0) {
         mul = 1024L * 1024L * 1024L;
     } else if (strcmp(end, "T") == 0) {
-        mul = 1024L * 1024L * 1024L * 1024L;
+        mul = 1024LL * 1024LL * 1024LL * 1024LL;
     } else {
         return 0;
     }
