@@ -18,10 +18,22 @@
 #   along with bindfs.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-require './common.rb'
+# if we are being run by make check it will set srcdir and we should use it
+$LOAD_PATH << (ENV['srcdir'] || '.')
+
+require 'common.rb'
+require 'etc'
 
 include Errno
 
+$have_fuse_29 = Proc.new do
+  v = `pkg-config --modversion fuse`.split('.')
+  raise "failed to get FUSE version with pkg-config" if v.size < 2
+  v = v.map(&:to_i)
+  v = [2, 8, 0]
+  v[0] > 2 || (v[0] == 2 && v[1] >= 9)
+end.call
+
 # FileUtils.chown turned out to be quite buggy in Ruby 1.8.7,
 # so we'll use File.chown instead.
 def chown(user, group, list)
@@ -38,8 +50,9 @@ end
 $nobody_uid = nobody_uid = Etc.getpwnam('nobody').uid
 $nobody_gid = nobody_gid = Etc.getpwnam('nobody').gid
 $nobody_group = nobody_group = Etc.getgrgid(nobody_gid).name
+$root_group = root_group = Etc.getgrgid(0).name
 
-$tests_dir = File.dirname(File.realpath(__FILE__))
+$tests_dir = File.realpath('.')
 
 
 testenv("") do
@@ -236,18 +249,20 @@ root_testenv("--mirror=root") do
     assert { File.stat('mnt/file').gid == $nobody_gid }
 end
 
-testenv("--chmod-allow-x --chmod-ignore") do
-    touch('src/file')
+if `uname`.strip != 'FreeBSD'  # FreeBSD doesn't let us set the sticky bit on files
+  testenv("--chmod-allow-x --chmod-ignore") do
+      touch('src/file')
 
-    chmod(01700, 'src/file') # sticky bit set
+      chmod(01700, 'src/file') # sticky bit set
 
-    chmod(00077, 'mnt/file') # should change x bits; should not unset sticky bit
-    assert { File.stat('src/file').mode & 07777 == 01611 }
+      chmod(00077, 'mnt/file') # should change x bits; should not unset sticky bit
+      assert { File.stat('src/file').mode & 07777 == 01611 }
 
-    mkdir('src/dir')
-    chmod(0700, 'src/dir')
-    chmod(0077, 'mnt/dir') # bits on dir should not change
-    assert { File.stat('src/dir').mode & 0777 == 0700 }
+      mkdir('src/dir')
+      chmod(0700, 'src/dir')
+      chmod(0077, 'mnt/dir') # bits on dir should not change
+      assert { File.stat('src/dir').mode & 0777 == 0700 }
+  end
 end
 
 testenv("--chmod-deny --chmod-allow-x") do
@@ -278,7 +293,7 @@ testenv("--chmod-filter=g-w,o-rwx") do
     assert { File.stat('src/file').mode & 0777 == 0640 }
 end
 
-root_testenv("--map=nobody/root:@#{nobody_group}/@root") do
+root_testenv("--map=nobody/root:@#{nobody_group}/@#{root_group}") do
     touch('src/file')
     chown('nobody', nobody_group, 'src/file')
 
@@ -299,7 +314,7 @@ root_testenv("--map=nobody/root:@#{nobody_group}/@root") do
     assert { File.stat('mnt/newdir').gid == 0 }
 end
 
-root_testenv("--map=@#{nobody_group}/@root") do
+root_testenv("--map=@#{nobody_group}/@#{root_group}") do
     touch('src/file')
     chown('nobody', nobody_group, 'src/file')
 
@@ -366,20 +381,21 @@ end
 root_testenv("--uid-offset=2 --gid-offset=20", :title => "file creation with --uid-offset and --gid-offset") do
     touch('mnt/file')
 
-    assert { File.stat('mnt/file').uid == 2 }
-    assert { File.stat('mnt/file').gid == 20 }
     assert { File.stat('src/file').uid == 0 }
-    assert { File.stat('src/file').gid == 0 }
+    assert { File.stat('mnt/file').uid == 2 }
+    # Note: BSDs tend to inherit group from parent dir while Linux uses the effective GID.
+    # This check works for both.
+    assert { File.stat('mnt/file').gid == File.stat('src/file').gid + 20 }
 end
 
 root_testenv("--uid-offset=2 --gid-offset=20", :title => "chown/chgrp with --uid-offset and --gid-offset") do
     touch('src/file')
     chown(6, 25, 'mnt/file')
 
-    assert { File.stat('mnt/file').uid == 6 }
-    assert { File.stat('mnt/file').gid == 25 }
     assert { File.stat('src/file').uid == 4 }
     assert { File.stat('src/file').gid == 5 }
+    assert { File.stat('mnt/file').uid == 6 }
+    assert { File.stat('mnt/file').gid == 25 }
 end
 
 testenv("", :title => "preserves inode numbers") do
@@ -389,7 +405,7 @@ testenv("", :title => "preserves inode numbers") do
     assert { File.stat('mnt/dir').ino == File.stat('src/dir').ino }
 end
 
-testenv("", :title => "has readdir inode numbers") do
+testenv("", :title => "preserves readdir inode numbers") do
     touch('src/file')
     mkdir('src/dir')
 
@@ -616,62 +632,70 @@ testenv("", :title => "many files in a directory") do
   assert { Dir.entries('mnt/dir').sort == expected_entries.sort }
 end
 
-testenv("--enable-lock-forwarding --multithreaded", :title => "lock forwarding") do
-  File.write('src/file', 'some contents for fcntl lockng')
-  # (this test passes with an empty file as well, but this way is clearer)
+if $have_fuse_29
+  testenv("--enable-lock-forwarding --multithreaded", :title => "lock forwarding") do
+    File.write('src/file', 'some contents for fcntl lockng')
+    # (this test passes with an empty file as well, but this way is clearer)
 
-  # flock
-  File.open('mnt/file') do |f1|
-    File.open('src/file') do |f2|
+    # flock
+    File.open('mnt/file') do |f1|
+      File.open('src/file') do |f2|
+        assert { f1.flock(File::LOCK_EX | File::LOCK_NB)  }
+        assert { !f2.flock(File::LOCK_EX | File::LOCK_NB)  }
+        assert { f1.flock(File::LOCK_UN)  }
+
+        assert { f2.flock(File::LOCK_EX | File::LOCK_NB)  }
+        assert { !f1.flock(File::LOCK_EX | File::LOCK_NB)  }
+      end
       assert { f1.flock(File::LOCK_EX | File::LOCK_NB)  }
-      assert { !f2.flock(File::LOCK_EX | File::LOCK_NB)  }
-      assert { f1.flock(File::LOCK_UN)  }
-
-      assert { f2.flock(File::LOCK_EX | File::LOCK_NB)  }
-      assert { !f1.flock(File::LOCK_EX | File::LOCK_NB)  }
     end
-    assert { f1.flock(File::LOCK_EX | File::LOCK_NB)  }
+
+    # fcntl locking
+    system("#{$tests_dir}/fcntl_locker src/file mnt/file")
+    raise "fcntl lock sharing test failed" unless $?.success?
   end
 
-  # fcntl locking
-  system("#{$tests_dir}/fcntl_locker src/file mnt/file")
-  raise "fcntl lock sharing test failed" unless $?.success?
-end
+  testenv("--disable-lock-forwarding", :title => "no lock forwarding") do
+    File.write('src/file', 'some contents for fcntl lockng')
 
-testenv("--disable-lock-forwarding", :title => "no lock forwarding") do
-  File.write('src/file', 'some contents for fcntl lockng')
+    # flock
+    File.open('mnt/file') do |f1|
+      File.open('src/file') do |f2|
+        assert { f1.flock(File::LOCK_EX | File::LOCK_NB)  }
+        assert { f2.flock(File::LOCK_EX | File::LOCK_NB)  }
+      end
+      File.open('mnt/file') do |f2|
+        assert { !f2.flock(File::LOCK_EX | File::LOCK_NB)  }
+      end
+    end
 
-  # flock
-  File.open('mnt/file') do |f1|
-    File.open('src/file') do |f2|
-      assert { f1.flock(File::LOCK_EX | File::LOCK_NB)  }
-      assert { f2.flock(File::LOCK_EX | File::LOCK_NB)  }
-    end
-    File.open('mnt/file') do |f2|
-      assert { !f2.flock(File::LOCK_EX | File::LOCK_NB)  }
-    end
+    # fcntl locking
+    system("#{$tests_dir}/fcntl_locker src/file mnt/file")
+    raise "fcntl lock sharing test failed" unless $?.exitstatus == 1
   end
-
-  # fcntl locking
-  system("#{$tests_dir}/fcntl_locker src/file mnt/file")
-  raise "fcntl lock sharing test failed" unless $?.exitstatus == 1
-end
+end # have_fuse_29
 
 # Issue #37
-# Valgrind disabled for ioctl tests since it seems to give a false negative
+#
+# Valgrind is disabled for ioctl tests since it seems to give a false negative
 # about a null parameter to ioctl.
-root_testenv("--enable-ioctl", :title => "append-only ioctl", :valgrind => false) do
-  touch('mnt/file')
-  system('chattr +a mnt/file')
-  raise 'chattr +a failed' unless $?.success?
-  begin
-    File.open('mnt/file', 'a') do |f|
-      f.write('stuff')
+#
+# This test is also disabled for old (FUSE < 2.9) systems.
+# TODO: figure out why it doesn't work.
+if $have_fuse_29
+  root_testenv("--enable-ioctl", :title => "append-only ioctl", :valgrind => false) do
+    touch('mnt/file')
+    system('chattr +a mnt/file')
+    raise 'chattr +a failed' unless $?.success?
+    begin
+      File.open('mnt/file', 'a') do |f|
+        f.write('stuff')
+      end
+      assert { File.read('mnt/file') == 'stuff' }
+      assert_exception(EPERM) { File.write('mnt/file', 'newstuff') }
+    ensure
+      system('chattr -a mnt/file')
     end
-    assert { File.read('mnt/file') == 'stuff' }
-    assert_exception(EPERM) { File.write('mnt/file', 'newstuff') }
-  ensure
-    system('chattr -a mnt/file')
   end
 end
 
@@ -692,9 +716,52 @@ testenv("", :title => "reading directory with rewind") do
   end
 end
 
+# Issue 47
+testenv("", :title => "srcdir with comma", :srcdir_name => "a,b") do
+  touch('a,b/f')
+  assert { File.exist?('mnt/f') }
+end
+
+testenv("", :title => "mntdir with comma", :mntdir_name => "a,b") do
+  touch('src/f')
+  assert { File.exist?('a,b/f') }
+end
+
+testenv("", :title => "srcdir with space", :srcdir_name => "a b") do
+  touch('a b/f')
+  assert { File.exist?('mnt/f') }
+end
+
+testenv("", :title => "mntdir with space", :mntdir_name => "a b") do
+  touch('src/f')
+  assert { File.exist?('a b/f') }
+end
+
+testenv("", :title => "srcdir with newline", :srcdir_name => "a\nb") do
+  touch("a\nb/f")
+  assert { File.exist?('mnt/f') }
+end
+
+testenv("", :title => "mntdir with newline", :mntdir_name => "a\nb") do
+  touch('src/f')
+  assert { File.exist?("a\nb/f") }
+end
+
+if `uname`.strip != 'FreeBSD'  # -o dev is not supported on FreeBSD
+  root_testenv("-odev") do
+    system("mknod mnt/zero c 1 5")
+    data = File.read("mnt/zero", 3)
+    assert { data.length == 3 }
+    assert { data.chars.to_a[0] == "\0" }
+    assert { data.chars.to_a[1] == "\0" }
+    assert { data.chars.to_a[2] == "\0" }
+  end
+end
+
 # FIXME: this stuff around testenv is a hax, and testenv may also exit(), which defeats the 'ensure' below.
 # the test setup ought to be refactored. It might well use MiniTest or something.
-if Process.uid == 0
+# TODO: support FreeBSD in this test (different group management commands)
+if Process.uid == 0 && `uname`.strip == 'Linux'
     begin
         `groupdel bindfs_test_group 2>&1`
         `groupadd -f bindfs_test_group`
